	list p=16f84
;**************************************************************
;*
;* Pinbelegung
;*	----------------------------------	
;*	PORTA: 	0 gemeinsames minus Grne leds
;*			1 gemeinsames minus Gelbe leds
;*			2 gemeinsames minus Rote leds
;*			3 Lautsprecher
;*			4 Vibrationswchter (Patsch-Schalter)
;*	PORTB:	0 LEDs
;*			1 LEDs
;*			2 LEDs
;*			3 LEDs
;*			4 LEDs
;*			5 LEDs
;*			6 LEDs
;*			7 LEDs
;*
;**************************************************************
;
;**************************************************************
; Includedatei fr den 16F84 einbinden

	#include <P16f84a.INC>

; Variablennamen vergeben

w_copy		Equ	0x20		; Backup fr Akkuregister
s_copy		Equ	0x21		; Backup fr Statusregister
LedsRot		Equ	0x22		; 8 LEDs
LedsGelb	Equ	0x23		; 
LedsGruen	Equ	0x24		; 
LedWer		Equ	0x26		; Welche LEDs sind gerade aktiv (3,2,1,0)
ar			Equ	0x27		; Zwischenspeicher
countf		Equ 0x28
loops2  	Equ 0x29
LiedNr		Equ 0x2A		; aktuelles Lied
Aktfrequ	Equ 0x2B		;
Aktlang		Equ 0x2C		;
AktposLed	Equ 0x2D		; Tabelle Position Led
AktposSong	Equ 0x2E		; Tabelle Position Song
Fertig		Equ 0x2F		; Fertig bei 0
Bitmerker	Equ	0x30		

#define	ARot		PORTA, 2
#define AGelb		PORTA, 1
#define AGruen		PORTA, 0
#define LS			PORTA, 3
#define Feder		PORTA, 4

#define Pause		Bitmerker,0
#define Neustart	Bitmerker,1
#define Langsamer	Bitmerker,2

;**************************************************************
; los gehts mit dem Programm

	org 	0
	goto	Init	

;**************************************************************
; die Interuptserviceroutine

	org 	4 	
intvec	
	bcf		INTCON, GIE	; disable Interupt 

	movwf	w_copy		; w retten
	swapf	STATUS, w 	; STATUS retten
	movwf	s_copy		;

	movlw	D'131'		; 256-125=131 ((1MHz : 32 ): 125 = 250 Hz)
	movwf	TMR0

; Intrupt servic routine
Int_serv 

;	movlw	B'00000000'  ;alle LEDs aus
;	movwf	PORTB

	decf	LedWer,f		; Ziffernzhler verringern

;LedWer=0: Rote LEDs aktiv und auf 3 setzen
;LedWer=2: Grne LEDs aktiv
;LedWer=1: Gelbe LEDs aktiv

	btfsc	STATUS, Z
	goto	Int_0		; Z-Flag=1 ergo LedWer=0
	movfw	LedWer
	movwf	ar
	decf	ar, f
	btfsc	STATUS, Z
	goto	Int_1		; LedWer=1
	decf	ar, f
	btfsc	STATUS, Z
	goto	Int_2		; LedWer=2

Int_0
	movlw	D'3'
	movwf	LedWer

	bsf		ARot
	bcf		AGelb
	bcf		AGruen	
	movfw	LedsRot
	movwf	PORTB
	goto	Int_w
Int_1
	bcf		ARot
	bsf		AGelb
	bcf		AGruen	
	movfw	LedsGelb
	movwf	PORTB
	goto	Int_w
Int_2
	bcf		ARot
	bcf		AGelb
	bsf		AGruen	
	movfw	LedsGruen
	movwf	PORTB
	goto	Int_w

Int_w
	btfsc	Langsamer		; Lied um die hlfte langsamer bitte
	goto	Langs
	bsf		Langsamer
	decfsz	Aktlang,F		; Tonlnge runterzhlen
	goto	Int_end			; Wenn vorbei dann neue Daten holen
	call 	NeuWert	        ; Neue Werte holen
	goto	Int_end	
Langs
	bcf		Langsamer

Int_end	
	swapf	s_copy, w	; STATUS zurck
	movwf	STATUS 
	swapf	w_copy, f	; w zurck mit flags
	swapf	w_copy, w

	bcf		INTCON, T0IF	; Interupt-Flag lschen
	bsf		INTCON, GIE	; enable Interupt

	retfie

;**************************************************************

; Port A/B auf Ausgabe stellen

Init
; PORTS einstellen nochmal
	movlw	B'00000000'
	movwf	PORTA
	movlw	B'00000000'
	movwf	PORTB		; Alles finster machen
	bsf     STATUS, RP0	; auf Bank 1 umschalten
	movlw	B'00010000'	; PortA alle output auer RA4
	movwf	TRISA
	movlw	B'00000000'	; PortB alle output
	movwf	TRISB
	bcf     STATUS, RP0	; auf Bank 0 zurckschalten

; Speicherwerte init
	movlw	D'3'
	movwf	LedWer
	movlw	B'00000001'
	movwf	LiedNr		; Start mit Lied Nr 1
	movwf	B'00000000'
	movwf	Bitmerker

; 250 Hz-Timer-Interupt einstellen
	bsf     STATUS, RP0	; auf Bank 1 umschalten
	movlw	B'10000100'	; internen Takt zhlen, Vorteiler zum Timer0, 32:1
	movwf	OPTION_REG
	movlw	D'131'		; 256-125=131 ((1MHz : 32 ): 125 = 250 Hz)
	bcf     STATUS, RP0	; auf Bank 0 zurckschalten
	movwf	TMR0
	bsf		INTCON, T0IE	; Timer0 interupt erlauben
;**********************************************************
;**********************************************************


Haupt
	bcf		INTCON, GIE	; disable Interupt 
; alles abdrehen und initialisieren
	movlw	B'00000000'
	movwf	LedsRot
	movwf	LedsGelb
	movwf	LedsGruen
	movwf	PORTB		
	movwf	PORTA	

	btfss	Neustart	;Beim einschalten mal ein Lied spielen...
	goto 	Fed2
Fed
	btfss	Feder       ;Solange Feder nicht bettigt passiert nix
	goto	Fed
Fed2
	bsf		Neustart
	movlw	B'00000011' ; 2 Bits fr LEDs und Song
	movwf	Fertig		;noch nicht fertig
	movlw	D'2'		;unhrbare Frequenz als erster Ton
	movwf	Aktfrequ	
	movlw	D'10'		;ganz kurze Tonlnge setzen
	movwf	Aktlang	

	btfsc	LiedNr,0
	goto	LiedLang1
	btfsc	LiedNr,1
	goto 	LiedLang2
	btfsc	LiedNr,2
	goto 	LiedLang3


LiedLang1
	movlw	HIGH TLED1	; HIGH-Adresse von Datentabelle
	movwf	PCLATH		; ins PCLatch-Register
	movlw	D'0'		; Gesamtlnge (Anzahl Datenstze+1) setzen
	call	TLED1
	movwf	AktposLed
	movlw	HIGH TSong1	; HIGH-Adresse von Datentabelle
	movwf	PCLATH		; ins PCLatch-Register
	movlw	D'0'		; Gesamtlnge (Anzahl Datenstze+1) setzen
	call	TSong1
	movwf	AktposSong
	movlw	HIGH Fed	;PCLATCH wiederherstellen
	movwf	PCLATH		
	goto	next
LiedLang2
	movlw	HIGH TLED2	; HIGH-Adresse von Datentabelle
	movwf	PCLATH		; ins PCLatch-Register
	movlw	D'0'		; Gesamtlnge (Anzahl Datenstze+1) setzen
	call	TLED2
	movwf	AktposLed
	movlw	HIGH TSong2	; HIGH-Adresse von Datentabelle
	movwf	PCLATH		; ins PCLatch-Register
	movlw	D'0'		; Gesamtlnge (Anzahl Datenstze+1) setzen
	call	TSong2
	movwf	AktposSong
	movlw	HIGH Fed	;PCLATCH wiederherstellen
	movwf	PCLATH		
	goto	next
LiedLang3
	movlw	HIGH TLED3	; HIGH-Adresse von Datentabelle
	movwf	PCLATH		; ins PCLatch-Register
	movlw	D'0'		; Gesamtlnge (Anzahl Datenstze+1) setzen
	call	TLED3
	movwf	AktposLed
	movlw	HIGH TSong3	; HIGH-Adresse von Datentabelle
	movwf	PCLATH		; ins PCLatch-Register
	movlw	D'0'		; Gesamtlnge (Anzahl Datenstze+1) setzen
	call	TSong3
	movwf	AktposSong
	movlw	HIGH Fed	;PCLATCH wiederherstellen
	movwf	PCLATH		
	goto	next
next
	bsf		INTCON, GIE	;Interrupt einschalten

Laut
	bsf		LS			;Lautsprecher ein
	movfw 	Aktfrequ 	;Faktor laden (*8s)
	movwf	countf		;auf counter schreiben
	call 	Wait		;und warten

	bcf		LS			;Lautsprecher aus
	movfw 	Aktfrequ 
	movwf	countf
	call	Wait	

	movf	Fertig,1	;in sich selbst kopieren
	btfss	STATUS, Z
	goto	Laut		;Wenn noch nicht fertig dann Lautsprecher

	bcf		STATUS,0	;Carry lschen
	rlf		LiedNr,1	;Nchstes Lied
	btfss	LiedNr,3	;Letztes Lied +1
	goto	Haupt		;Zurck zum Hauptprogramm
	movlw	B'00000001'
	movwf	LiedNr		; Start mit Lied Nr 1
	goto	Haupt	

;**********************************************************
; Warteschleife Frequenz: Vorher muss der Faktor (aktuelle Frequenz) in countf geladen werden
Wait
    nop                    ; 1 s
    nop
    nop
    nop
    nop
    decfsz  countf, F      ; x mal 8s vorbei?
    goto    Wait           ; nein, noch nicht
    retlw   0              ; das Warten hat ein Ende

;Neuen Wert aus Datentabellen holen
NeuWert
	btfss	Pause		
	goto	NeuWert1

	bcf		Pause		;Pause lschen
	movlw	D'2'		;unhrbare Frequenz als minipause
	movwf	Aktfrequ	
	movlw	D'4'		;ganz kurze Tonlnge setzen (64tel)
	movwf	Aktlang	
	RETURN

NeuWert1
	bsf		Pause		;Pause fr nchstes mal setzen

	btfsc	LiedNr,0
	goto	Lied1
	btfsc	LiedNr,1
	goto 	Lied2
	btfsc	LiedNr,2
	goto 	Lied3

Lied1
	movf	AktposLed,1	
	btfsc	STATUS, Z
	bcf		Fertig, 0
	btfss	Fertig, 0
	goto	Song1		

	movlw	HIGH TLED1		; HIGH-Adresse von Datentabelle
	movwf	PCLATH			; ins PCLatch-Register
	movfw	AktposLed		; aus Tabelle
	call	TLED1
	movwf	LedsGruen		
	decf	AktposLed,f		; nchste Position
	movfw	AktposLed		; aus Tabelle
	call	TLED1
	movwf	LedsGelb		
	decf	AktposLed,f		; nchste Position
	movfw	AktposLed		; aus Tabelle
	call	TLED1
	movwf	LedsRot		
	decf	AktposLed,f		; nchste Position
Song1
	movf	AktposSong,1	
	btfsc	STATUS, Z
	bcf		Fertig, 1
	btfss	Fertig, 1
	RETURN		

	movlw	HIGH TSong1		; HIGH-Adresse von Datentabelle
	movwf	PCLATH			; ins PCLatch-Register
	movfw	AktposSong		; aus Tabelle
	call	TSong1
	movwf	Aktlang	
	decf	AktposSong,f
	movfw	AktposSong		; aus Tabelle
	call	TSong1
	movwf	Aktfrequ	
	decf	AktposSong,f

	movlw	HIGH NeuWert ;
	movwf	PCLATH			;PCLATCH wiederherstellen
	RETURN

Lied2
	movf	AktposLed,1	
	btfsc	STATUS, Z
	bcf		Fertig, 0
	btfss	Fertig, 0
	goto	Song2		

	movlw	HIGH TLED2		; HIGH-Adresse von Datentabelle
	movwf	PCLATH			; ins PCLatch-Register
	movfw	AktposLed		; aus Tabelle
	call	TLED2
	movwf	LedsGruen		
	decf	AktposLed,f		; nchste Position
	movfw	AktposLed		; aus Tabelle
	call	TLED2
	movwf	LedsGelb		
	decf	AktposLed,f		; nchste Position
	movfw	AktposLed		; aus Tabelle
	call	TLED2
	movwf	LedsRot		
	decf	AktposLed,f		; nchste Position
Song2
	movf	AktposSong,1	
	btfsc	STATUS, Z
	bcf		Fertig, 1
	btfss	Fertig, 1
	RETURN		

	movlw	HIGH TSong2		; HIGH-Adresse von Datentabelle
	movwf	PCLATH			; ins PCLatch-Register
	movfw	AktposSong		; aus Tabelle
	call	TSong2
	movwf	Aktlang	
	decf	AktposSong,f
	movfw	AktposSong		; aus Tabelle
	call	TSong2
	movwf	Aktfrequ	
	decf	AktposSong,f

	movlw	HIGH NeuWert ;
	movwf	PCLATH			;PCLATCH wiederherstellen
	RETURN

Lied3
	movf	AktposLed,1	
	btfsc	STATUS, Z
	bcf		Fertig, 0
	btfss	Fertig, 0
	goto	Song3		

	movlw	HIGH TLED3		; HIGH-Adresse von Datentabelle
	movwf	PCLATH			; ins PCLatch-Register
	movfw	AktposLed		; aus Tabelle
	call	TLED3
	movwf	LedsGruen		
	decf	AktposLed,f		; nchste Position
	movfw	AktposLed		; aus Tabelle
	call	TLED3
	movwf	LedsGelb		
	decf	AktposLed,f		; nchste Position
	movfw	AktposLed		; aus Tabelle
	call	TLED3
	movwf	LedsRot		
	decf	AktposLed,f		; nchste Position
Song3
	movf	AktposSong,1	
	btfsc	STATUS, Z
	bcf		Fertig, 1
	btfss	Fertig, 1
	RETURN		

	movlw	HIGH TSong3		; HIGH-Adresse von Datentabelle
	movwf	PCLATH			; ins PCLatch-Register
	movfw	AktposSong		; aus Tabelle
	call	TSong3
	movwf	Aktlang	
	decf	AktposSong,f
	movfw	AktposSong		; aus Tabelle
	call	TSong3
	movwf	Aktfrequ	
	decf	AktposSong,f

	movlw	HIGH NeuWert ;
	movwf	PCLATH			;PCLATCH wiederherstellen
	RETURN



	org		0x180
TLED1
	addwf	PCL,f
	retlw	D'126' ;gesamtanzahl datenstze
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'11111111',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'11111111',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000001',B'00000000'
	dt B'00000000',B'00000000',B'00000001'
	dt B'00000001',B'00000000',B'00000000'
	dt B'00000000',B'00000010',B'00000000'
	dt B'00000000',B'00000000',B'00000010'
	dt B'00000010',B'00000000',B'00000000'
	dt B'00000000',B'00000100',B'00000000'
	dt B'00000000',B'00000000',B'00000100'
	dt B'00000100',B'00000000',B'00000000'
	dt B'00000000',B'00001000',B'00000000'
	dt B'00000000',B'00000000',B'00001000'
	dt B'00001000',B'00000000',B'00000000'
	dt B'00000000',B'00010000',B'00000000'
	dt B'00000000',B'00000000',B'00010000'
	dt B'00010000',B'00000000',B'00000000'
	dt B'00000000',B'00100000',B'00000000'
	dt B'00000000',B'00000000',B'00100000'
	dt B'00100000',B'00000000',B'00000000'
	dt B'00000000',B'01000000',B'00000000'
	dt B'00000000',B'00000000',B'01000000'
	dt B'01000000',B'00000000',B'00000000'
	dt B'00000000',B'10000000',B'00000000'
	dt B'00000000',B'00000000',B'10000000'
	dt B'10000000',B'00000000',B'00000000'

	org 0x200
TSong1
	addwf	PCL, f
	retlw	D'84' ;gesamtanzahl datenstze
	dt D'239',D'125'
	dt D'213',D'63'
	dt D'213',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'179',D'63'
	dt D'179',D'63'
	dt D'159',D'125'
	dt D'142',D'63'
	dt D'142',D'63'
	dt D'159',D'63'
	dt D'159',D'63'
	dt D'239',D'63'
	dt D'239',D'63'
	dt D'213',D'125'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'179',D'63'
	dt D'179',D'63'
	dt D'159',D'63'
	dt D'159',D'63'
	dt D'213',D'125'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'179',D'63'
	dt D'179',D'63'
	dt D'159',D'63'
	dt D'159',D'63'
	dt D'239',D'125'
	dt D'213',D'63'
	dt D'213',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'179',D'63'
	dt D'179',D'63'
	dt D'159',D'125'
	dt D'142',D'63'
	dt D'142',D'63'
	dt D'159',D'63'
	dt D'159',D'63'
	dt D'239',D'63'
	dt D'239',D'63'

	org		0x260
TLED2
	addwf	PCL,f
	retlw	D'129' ;gesamtanzahl datenstze
	dt B'11111111',B'11111111',B'11111111'
	dt B'11111111',B'11111111',B'11111111'
	dt B'11111111',B'11111111',B'11111111'
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'11111111',B'11111111',B'11111111'
	dt B'11111111',B'11111111',B'11111111'
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'11111111',B'11111111',B'11111111'
	dt B'11111111',B'11111111',B'11111111'
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'10000000',B'10000000',B'10000000'
	dt B'01000000',B'01000000',B'01000000'
	dt B'00100000',B'00100000',B'00100000'
	dt B'00010000',B'00010000',B'00010000'
	dt B'00001000',B'00001000',B'00001000'
	dt B'00000100',B'00000100',B'00000100'
	dt B'00000010',B'00000010',B'00000010'
	dt B'00000001',B'00000001',B'00000001'
	dt B'10000000',B'10000000',B'10000000'
	dt B'01000000',B'01000000',B'01000000'
	dt B'00100000',B'00100000',B'00100000'
	dt B'00010000',B'00010000',B'00010000'
	dt B'00001000',B'00001000',B'00001000'
	dt B'00000100',B'00000100',B'00000100'
	dt B'00000010',B'00000010',B'00000010'
	dt B'00000001',B'00000001',B'00000001'
	dt B'10000000',B'10000000',B'10000000'
	dt B'01000000',B'01000000',B'01000000'
	dt B'00100000',B'00100000',B'00100000'
	dt B'00010000',B'00010000',B'00010000'
	dt B'00001000',B'00001000',B'00001000'
	dt B'00000100',B'00000100',B'00000100'
	dt B'00000010',B'00000010',B'00000010'
	dt B'00000001',B'00000001',B'00000001'

	org 0x300
TSong2
	addwf	PCL, f
	retlw	D'86' ;gesamtanzahl datenstze
	dt D'239',D'125'
	dt D'213',D'63'
	dt D'213',D'63'
	dt D'190',D'63'
	dt D'179',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'179',D'125'
	dt D'159',D'63'
	dt D'142',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'239',D'63'
	dt D'213',D'125'
	dt D'190',D'63'
	dt D'179',D'63'
	dt D'239',D'63'
	dt D'213',D'63'
	dt D'239',D'63'
	dt D'2',D'63'
	dt D'213',D'125'
	dt D'190',D'63'
	dt D'179',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'179',D'125'
	dt D'159',D'63'
	dt D'142',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'190',D'63'
	dt D'239',D'63'
	dt D'213',D'125'
	dt D'190',D'63'
	dt D'179',D'63'
	dt D'239',D'63'
	dt D'213',D'63'
	dt D'239',D'63'


;	org		0x300
TLED3
	addwf	PCL,f
	retlw	D'84' ;gesamtanzahl datenstze
	dt B'11111111',B'11111111',B'11111111'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'00000000',B'00000000',B'00000000'
	dt B'10000000',B'10000000',B'10000000'
	dt B'01000000',B'01000000',B'01000000'
	dt B'00100000',B'00100000',B'00100000'
	dt B'00010000',B'00010000',B'00010000'
	dt B'00001000',B'00001000',B'00001000'
	dt B'00000100',B'00000100',B'00000100'
	dt B'00000010',B'00000010',B'00000010'
	dt B'00000001',B'00000001',B'00000001'
	dt B'10000000',B'10000000',B'10000000'
	dt B'01000000',B'01000000',B'01000000'
	dt B'00100000',B'00100000',B'00100000'
	dt B'00010000',B'00010000',B'00010000'
	dt B'00001000',B'00001000',B'00001000'
	dt B'00000100',B'00000100',B'00000100'
	dt B'00000010',B'00000010',B'00000010'
	dt B'00000001',B'00000001',B'00000001'
	dt B'10000000',B'10000000',B'10000000'
	dt B'01000000',B'01000000',B'01000000'
	dt B'00100000',B'00100000',B'00100000'
	dt B'00010000',B'00010000',B'00010000'
	dt B'00001000',B'00001000',B'00001000'
	dt B'00000100',B'00000100',B'00000100'
	dt B'00000010',B'00000010',B'00000010'
	dt B'00000001',B'00000001',B'00000001'

TSong3
	addwf	PCL, f
	retlw	D'56' ;gesamtanzahl datenstze
	dt D'2',D'250'
	dt D'119',D'250'
	dt D'106',D'63'
	dt D'106',D'63'
	dt D'106',D'63'
	dt D'106',D'63'
	dt D'95',D'125'
	dt D'95',D'125'
	dt D'89',D'63'
	dt D'89',D'63'
	dt D'89',D'63'
	dt D'89',D'63'
	dt D'80',D'250'
	dt D'71',D'63'
	dt D'71',D'63'
	dt D'71',D'63'
	dt D'71',D'63'
	dt D'80',D'250'
	dt D'71',D'63'
	dt D'71',D'63'
	dt D'71',D'63'
	dt D'71',D'63'
	dt D'80',D'125'
	dt D'80',D'125'
	dt D'89',D'63'
	dt D'95',D'63'
	dt D'106',D'63'
	dt D'119',D'63'

	end
